#!/usr/bin/perl
# ----------------------------------------------------------------------
# Copyright (C) 2001 Bodo Bauer <bb@bb-zone.com>
#
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details. 
#
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
# MA 02111-1307, USA. 
# ----------------------------------------------------------------------
#
# Gimp Perl script to scale a batch of images to given size. The size
# give is compared to the bigger edge of the original and the resulting
# factor is applied accordingly to the smaller edge to preserve the
# aspect ratio. 
#
# Images are scipped if a newer (than the original) output file is
# detected.
# ----------------------------------------------------------------------
# Changelog
#
# 2001-06-06   BB   Initial release
#
# ----------------------------------------------------------------------
use Gimp ":auto"; 
use Gimp::Fu;
use strict;

sub scale_all () {
    my $img_dir = shift;
    my $out_dir = shift;
    my $prefix  = shift;
    my $size    = shift;
    my $verbose = shift;
    my $force   = shift;
    my $quality = shift() / 100;

    my $cwd;
    chomp($cwd = `pwd`);

    $img_dir = "$cwd/$img_dir" if (  $img_dir !~ /^\// );
    $out_dir = "$cwd/$out_dir" if ( $out_dir  !~ /^\// );

    $|=1;

    print "Reading images in $img_dir\n" if ( $verbose );
    if ( opendir ( IN, $img_dir ) ) {
	my @images = sort ( readdir ( IN ) );
	close ( IN );
	
	for ( my $i=0; $i<=$#images; $i++ ) {
	    if ( $images[$i] =~ /.*\.jpg$/ ) {
		print "  $images[$i] " if ( $verbose );

		my $infile = "$img_dir/$images[$i]";
		my $outfile = "$out_dir/$prefix$images[$i]";
		
		my $skip=0;
		if ( -f $infile && ( my $inmtime = (stat(_))[9] ) ) {
		    if ( -f $outfile && ( my $outmtime = (stat(_))[9] ) ) {
			$skip = 1 if ( $inmtime < $outmtime );
		    }
		}

		if ( $skip && !$force ) {
		    print "skipping\n" if ( $verbose );
		} else {
		    print "loading " if ( $verbose );
		    my $img = file_jpeg_load ( "$img_dir/$images[$i]", "$images[$i]" );
		    my $h = gimp_image_height ( $img );
		    my $w = gimp_image_width ( $img );
		    
		    my $nh;
		    my $nw;
		    if ( $h>$w ) {
			$nh = $size;
			$nw = ( $size/$h ) * $w;
		    } else {
			$nw = $size;
			$nh = ( $size/$w ) * $h;
		    }
		    print "scaling " if ( $verbose ); 
		    gimp_image_scale ( $img, $nw, $nh );
		    
		    print "saving " if ( $verbose ); 
		    my $drw = gimp_image_active_drawable ( $img );
		    file_jpeg_save ( $img, $drw, "$out_dir/$prefix$images[$i]", "$prefix$images[$i]",
				     $quality, 0, 0, 0, "Created by the GIMP", 0, 0, 0, 0 );
		    
		    gimp_image_delete ( $img );
		    print "done\n" if ( $verbose );
		}
	    }
	}
    }
    return ();
}

register 
    "bb_scale_all",                
    "Scale all images in a directory",  
    "Used by BBGallery",
    "Bodo Bauer",            
    "Bodo Bauer (c)",        
    "2001-06-06",            
    "<Toolbox>/Xtns/Perl-Fu/BB/Scale All",  
    "*",                       
    [ 
      [PF_STRING, "img_dir", "Image Directory",  "."], 
      [PF_STRING, "out_dir", "Output_directory", "."] ,
      [PF_STRING, "prefix",  "Prefix for scaled images", "scaled_"] ,
      [PF_INT,    "size",    "New Size", "800"] ,
      [PF_TOGGLE, "verbose", "Be verbose?", "1"],
      [PF_TOGGLE, "force",   "Fore scaling?", "0"],
      [PF_INT,    "quality", "Image Quality", "100"] ,
      ], 
    \&scale_all; 

exit main(); 








