#!/usr/bin/perl -w
# ----------------------------------------------------------------------
# Copyright (C) 2001 Bodo Bauer <bb@bb-zone.com>
#
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details. 
#
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
# MA 02111-1307, USA. 
#
# ----------------------------------------------------------------------
# Simple script to generate html based galleries
# ----------------------------------------------------------------------
use strict;
use HTML::Entities;
use URI::Escape;

#
# Default parameters
#
my $P = {
    sc_size      => 800,        # Size of scaled images
    th_size      => 100,        # size of thumb nails
    scale        => 1,          # scale images?
    cols         => 5,          # columns on index pages
    th_max       => 25,         # max thumb nails per index page
    title        => 'Gallery',  # title for index pages
    img_dir      => '.',        # directory with originals
    verbose      => 0,          # be verbose?
    force        => 0,          # force thumbnail and scales creation
    slide_delay  => 15,         # delay between two slides (sec)
    meta         => 0,          # create meta index
    metalink     => 0,          # create link to meta index in galleries
    gimp         => 1,          # use Gimp or ImageMagick to scale?
    quality      => 100,        # JPEG quality for scaled images
    raw_captions => 0,          # omit underscore replacement for image captions?
    short_dsc    => 1,          # enable short descriptions?
    thumb_only   => 0,          # thumbnails instead of scale?
    no_save      => 0,          # dont save config if set to 1

    # page colors
    bgcolor  => "#999999",
    link     => "#FF0000",
    vlink    => "#0000FF",
    alink    => "#FF0000",
    text     => "#FFFFFF",
    text1    => "#FF0000",  # gallery titles in meta index
    text2    => "#FFFFFF",  # nr. of images in meta index

    bgcolor1 => "#666666",  # background for table cells
    bgcolor2 => "#AAAAAA",  # background for tables
};

my %P = %{$P};
my $version    = "%%%VERSION%%%";    # Version Number
my $instprefix = "%%%INSTPREFIX%%%";

# ----------------------------------------------------------------------
#                     S U B R O U T I N E S
# ----------------------------------------------------------------------

sub strip_ws ( $ ) {
    my $string = shift;
    if ( defined ( $string ) ) {
        for ($string) { s/^\s+//; s/\s+$//; }
	return ( $string );
    }
    return ( undef );
}

sub load_config ( $$ ) {
    my $cnf  = shift;
    my $file = shift;
    if ( open ( CNF, $file ) ) {
	print "Reading config from $file\n" if ( $P{verbose} );
	while ( <CNF> ) {
	    next if ( /^\s*$/ || /^\s*\#/ );
	    my ( $token, $value ) = split ( /=/, $_, 2 );
	    $token = strip_ws ( $token );
	    $value = strip_ws ( $value );
	    
	    for ( $token ) {
		if ( defined ( $P{$token} ) ) {
		    $cnf->{$token} = $value;
		} else {
		    print "Warning: Unknown token '$token' in config file: $file\n";
		}
	    }
	}
	close ( CNF );
    } else {
	print "Warning: Can not read config file: $file ($!)"  if ( $P{verbose} );
    }
}

sub write_config ( $ ) {
    my $file = shift;
    return ( 1 ) if ( $P{no_save} );

    if ( open ( CNF, ">$file" ) ) {
	print CNF "#-----------------------------------------------------------------------\n";
        print CNF "# BBGallery settings - v$version\n";
	print CNF "#-----------------------------------------------------------------------\n";

	foreach my $var ( keys  ( %P ) ) {
	    next if ( $var eq 'force' );
	    next if ( $var eq 'no_save' );
	    next if ( $var eq 'img_dir' );
	    print CNF "$var = $P{$var}\n";
	}
	print CNF "# END\n";
	close ( CNF );
	return ( 1 );
    } else {
	print "Warning: Could not save configuration to $file ($!)\n" if ( $P{verbose} );
	return ( 0 );
    }
}

sub im_scale ( $$$$$$$ ) {
    my $in_dir  = shift;
    my $out_dir = shift;
    my $prefix  = shift;
    my $size    = shift;
    my $verbose = shift;
    my $force   = shift;
    my $quality = shift;

    my $cwd;
    chomp($cwd = `pwd`);

    $P{img_dir} = "$cwd/$P{img_dir}" if (  $P{img_dir} !~ /^\// );
    $out_dir = "$cwd/$out_dir" if ( $out_dir  !~ /^\// );

    $|=1;

    print "Reading images in $P{img_dir}\n" if ( $verbose );
    if ( opendir ( IN, $P{img_dir} ) ) {
	my @images = sort ( readdir ( IN ) );
	close ( IN );
	
	for ( my $i=0; $i<=$#images; $i++ ) {
	    if ( $images[$i] =~ /.*\.jpg$/ ) {
		print "  $images[$i] " if ( $verbose );

		my $infile = "$P{img_dir}/$images[$i]";
		my $outfile = "$out_dir/$prefix$images[$i]";
		
		my $skip=0;
		if ( -f $infile && ( my $inmtime = (stat(_))[9] ) ) {
		    if ( -f $outfile && ( my $outmtime = (stat(_))[9] ) ) {
			$skip = 1 if ( $inmtime < $outmtime );
		    }
		}

		if ( $skip && !$force ) {
		    print "skipping\n" if ( $verbose );
		} else {
		    print "loading " if ( $verbose );
		    
		    open ( ID, "identify -format \"%w %h\" $P{img_dir}/$images[$i]|" ) || die ( "identify not found ($!)\n" );
		    my ($h, $w ) = split ( /\s+/, <ID> , 2 );
		    close ( ID );

		    print "$h x $w " if ( $verbose );

		    my $nh;
		    my $nw;
		    
		    if ( $h && $w ) {
			if ( $h>$w ) {
			    $nh = $size;
			    $nw = int ( ( $size/$h ) * $w );
			} else {
			    $nw = $size;
			    $nh = int ( ( $size/$w ) * $h );
			}
			print "scaling to $nh x $nw " if ( $verbose ); 
			system ( "convert $P{img_dir}/$images[$i] -quality $quality -sample $nw" . "x" . "$nh JPEG:$out_dir/$prefix$images[$i]" );
			print "done\n" if ( $verbose );
		    } else {
			print "Corrupt image file ($images[$i]) skipping ...";
		    }
		}
	    }
	}
    }
}

sub scale_images () {
   mkdir ( "$P{img_dir}/scaled", 0777 );

   if ( $P{gimp} ) {
       system ( "$instprefix/lib/bbgallery/gimp_scale.pl -img_dir $P{img_dir} -out_dir $P{img_dir}/scaled " .
		"-prefix sc_ -size $P{sc_size} -verbose $P{verbose} -force $P{force} -quality $P{quality}"  );
   } else {
       im_scale ( $P{img_dir}, "$P{img_dir}/scaled", "sc_", $P{sc_size}, $P{verbose}, $P{force}, $P{quality} );
   }
}

sub create_thumbs () {
   mkdir ( "$P{img_dir}/thumbs", 0777 );
   if ( $P{gimp} ) {
       system ( "$instprefix/lib/bbgallery/gimp_scale.pl -img_dir $P{img_dir} -out_dir $P{img_dir}/thumbs " .
		"-prefix th_ -size $P{th_size} -verbose $P{verbose} -force $P{force} -quality $P{quality}"  );
   } else {
       im_scale ( $P{img_dir},  "$P{img_dir}/thumbs", "th_", $P{th_size}, $P{verbose}, $P{force}, $P{quality} );
   }
}

sub meta_index_entry ( $ ) {
    my $img = shift;
    my $nr_img = $#{$img} + 1;
    if ( open ( MI, ">$P{img_dir}/.bbmeta" ) ) {
	my @d = split ( /\//, $P{img_dir} );
	my $subdir = $d[$#d];
	my $subdir_html = encode_entities($subdir);
 	my $subdir_uri  = encode_entities(uri_escape($subdir));
 	my $thumb_uri   = encode_entities(uri_escape($img->[0]->{THUMB}));
	print MI<<END;
  <TD ALIGN="CENTER" VALIGN="MIDDLE">
    <FONT size="-1" COLOR="$P{text1}">$P{title}</FONT><BR>
    <A HREF="$subdir_uri/index_0.html"><IMG SRC="$subdir_uri/thumbs/$thumb_uri" ALT="$subdir" BORDER="1"><BR></A>
    <FONT size="-2" COLOR="$P{text2}">($nr_img images)</FONT>
  </TD>
END
	close ( MI );
    }
}

sub create_meta_index () {
    print "Creating meta index\n" if ( $P{verbose} );

    if ( opendir ( DIR, $P{img_dir} ) ) {
	my @dir = sort ( readdir ( DIR ) );
	close ( DIR );
	if ( open ( MI, ">$P{img_dir}/index.html" ) ) {
	    print MI<<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
<TITLE>$P{title}</TITLE>
</HEAD>

<!--  Created by BBGallery $version                            -->
<!--  http://www.bb-zone.com/zope/bbzone/projects/bbgallery    -->

<BODY BGCOLOR="$P{bgcolor}" LINK="$P{link}" VLINK="$P{vlink}" ALINK="$P{alink}" TEXT="$P{text}">
<CENTER>
<H1>$P{title}</H1>
<P>
<TABLE BGCOLOR="$P{bgcolor2}"><TR><TD>
<TABLE BGCOLOR="$P{bgcolor1}" CELLPADDING="5">
  <TR>
END
            my $cnt = 0;
	    for ( my $i=0; $i<=$#dir; $i++ ) {
		if ( $dir[$i] !~ /^\./ && -d "$P{img_dir}/$dir[$i]" && -f "$P{img_dir}/$dir[$i]/.bbmeta" ) {
		    print "Adding gallery in $dir[$i]\n" if ( $P{verbose} );
		    if ( $cnt && $cnt % $P{cols} == 0 ) {
			print MI "  </TR>\n  <TR>\n";
		    }
		    if ( open ( ENTRY, "$P{img_dir}/$dir[$i]/.bbmeta" ) ) {
			while ( <ENTRY> ) {
			    print MI $_;
			}
			close ( ENTRY );
			$cnt++;
		    }
		}
	    }
	    print MI<<END;
  </TR>
</TABLE>
</TD></TR></TABLE>
<HR>
<FONT SIZE="-2">Created by <A HREF="http://www.bb-zone.com/zope/bbzone/projects/bbgallery">BBGallery</A></FONT>
</CENTER>
</BODY>
</HTML>
END
	}
    }
}

sub image_page ( $$$$ ) {
    my $img        = shift;
    my $img_idx    = shift;
    my $index      = shift;
    my $slide      = shift;
    my $image      = $img->[$img_idx]->{THUMB};
       $image      =~ s/^th_(.*)\.jpg/$1/;
    my $image_cap  = $image;
    if ( ! $P{raw_captions} ) {
	$image_cap  =~ s/_/ /g;
    } else {
	$image_cap  = $image_cap . '.jpg';
    }
    $image_cap     =  encode_entities($image_cap);
    my $image_uri  = encode_entities(uri_escape($image));
    my $filename   = ( $slide ) ? ">$P{img_dir}/sl_$image.html" : ">$P{img_dir}/$image.html";
    my $jpheader = '';
    if ( open ( HDR, "$P{img_dir}/header.txt" ) ) {
	while ( <HDR> ) { $jpheader .= $_; }
	close ( HDR );
    }
    if ( open ( IMG, $filename ) ) {
	print "writing $filename\n" if ( $P{verbose} );

	my $dsc   = '';
	my $short = '';
	if ( -f "$P{img_dir}/$image.txt" ) {
	    if ( open ( DSC, "$P{img_dir}/$image.txt" ) ) {
		print "Reading image description\n" if ( $P{verbose} );
		$short = <DSC>;
		$dsc = $short;
		chomp($short);
		while ( <DSC> ) {
		    $dsc .= $_;
		}
		close ( DSC );
	    }
	    if ( $dsc ne '' ) {
		$dsc = "<P>$dsc";
	    }
	}
	$short =~ s/\<[^\>]*\>//g;

	my $next_idx = ( $img_idx == $#{$img} ) ? -1 : $img_idx + 1;
	my $prev_idx = ( $img_idx ) ? $img_idx-1 : -1;
	my $this = ( $img_idx + 1 ) . ' / ' . ($#{$img}+1);
	
	my $d = $img->[$next_idx]->{THUMB};
	$d =~ s/^th_(.*)\.jpg/$1/;
	my $d_html   = encode_entities($d);
 	my $d_uri    = encode_entities(uri_escape($d));
	my $next     = ( $next_idx < 0 ) ? '&nbsp;' : "<A HREF=\"$d_uri.html\">Next</A>";
	my $next_th  = ( $next_idx < 0 ) ? '&nbsp;' : "<A HREF=\"$d_uri.html\"><IMG SRC=\"thumbs/th_$d_uri.jpg\" BORDER=\"1\" ALT=\"$d_html\"></A>";
	my $next_hdr = ( $next_idx < 0 ) ? '' : "<LINK rel=\"next\" type=\"text/html\" HREF=\"$d_uri.html\">";

	if ( $next_idx < 0 ) {
	    $d = $img->[0]->{THUMB};
	    $d =~ s/^th_(.*)\.jpg/$1/;
	    $d_html  = encode_entities($d);
	    $d_uri   = encode_entities(uri_escape($d));
	}
	my $meta       = ( $slide ) ? "<META HTTP-EQUIV=\"Refresh\" CONTENT=\"$P{slide_delay};URL=sl_$d_uri.html\">": '';

	$d             = $img->[$prev_idx]->{THUMB};
	$d             =~ s/^th_(.*)\.jpg/$1/;
	$d_html        = encode_entities($d);
	$d_uri         = encode_entities(uri_escape($d));

	my $jp         = encode_entities(uri_escape($img->[$img_idx]->{THUMB}));
	my $this_th    = "<IMG SRC=\"thumbs/$jp\" ALT=\"$short\" TITLE=\"$short\" BORDER=\"1\">";

	my $prev       = ( $prev_idx < 0 ) ? '&nbsp;' : "<A HREF=\"$d_uri.html\">Previous</A>";
	my $prev_th    = ( $prev_idx < 0 ) ? '&nbsp;' : "<A HREF=\"$d_uri.html\"><IMG SRC=\"thumbs/th_$d_uri.jpg\" BORDER=\"1\" ALT=\"$d_html\"></A>";
	my $prev_hdr   = ( $prev_idx < 0 ) ? '' : "<LINK rel=\"prev\" type=\"text/html\" HREF=\"$d_uri.html\">";
	my $slide_link = ( ! $slide ) ? "<A HREF=\"sl_$image_uri.html\">Start Slideshow<A>" :  "<A HREF=\"$image_uri.html\">Stop Slideshow<A>";
	my $ml_hdr     = ( $P{metalink} ) ? "<link rel=\"start\" type=\"text/html\" HREF=\"../index.html\">" : '';

	print IMG<<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
<TITLE>$P{title}: $image_cap</TITLE>
$meta
$ml_hdr
$prev_hdr
$next_hdr
<LINK rel="up" type="text/html" href="index_$index.html">
$jpheader
</HEAD>

<!--  Created by BBGallery $version                            -->
<!--  http://www.bb-zone.com/zope/bbzone/projects/bbgallery    -->

<BODY BGCOLOR="$P{bgcolor}" LINK="$P{link}" VLINK="$P{vlink}" ALINK="$P{alink}" TEXT="$P{text}">
<TABLE WIDTH="100%" COLS="3">
  <TR>
    <TD ALIGN="LEFT">$prev</TD>
    <TD ALIGN="CENTER" VALIGN="MIDDLE"><A HREF="index_$index.html">Index</A><P>$this</TD>
    <TD ALIGN="RIGHT">$next</TD>
  </TR>
</TABLE>
<HR>
<CENTER>
<TABLE>
  <TR>
    <TD>$prev_th</TD>
END
      if ( $P{thumb_only}) {
	  print IMG<<END;
 <TD ALIGN="CENTER" VALIGN="MIDDLE">
     <A HREF="$image.jpg">$this_th</A><BR>
       <FONT SIZE="-1">$image.jpg</FONT>
 </TD>
END
} else {
        if ( $P{scale} ) {
	    print IMG<<END;
    <TD ALIGN="CENTER" VALIGN="MIDDLE">
      <A HREF="$image_uri.jpg"><IMG SRC="scaled/sc_$image_uri.jpg" BORDER="1" ALT="$image_cap"></A><BR>
      <FONT SIZE="-1">$image_cap</FONT>
    </TD>
END
        } else {
	    print IMG<<END;
    <TD ALIGN="CENTER" VALIGN="MIDDLE">
      <IMG SRC="$image_uri.jpg" BORDER="0" ALT="$image_cap"><BR>
      <FONT SIZE="-1">$image_cap</FONT>
    </TD>
END
	}
    }
	print IMG<<END;
    <TD>$next_th</TD>
  </TR>
</TABLE>
</CENTER>
<BLOCKQUOTE>$dsc</BLOCKQUOTE>
<HR>
<TABLE WIDTH="100%" COLS="3">
  <TR>
    <TD ALIGN="LEFT">$prev</TD>
    <TD ALIGN="CENTER" VALIGN="MIDDLE">$this<P>$slide_link</A></TD>
    <TD ALIGN="RIGHT">$next</TD>
  </TR>
</TABLE>
<CENTER>
<HR>
<FONT SIZE="-2">Created by <A HREF="http://www.bb-zone.com/zope/bbzone/projects/bbgallery">BBGallery</A></FONT>
</CENTER>
</BODY>
</HTML>
END
	close ( IMG );
    }
}

sub index_page ( $$$$$$ ) {
    my $range = shift;
    my $img   = shift;
    my $prev  = shift;
    my $this  = shift;
    my $next  = shift;
    my $total = shift;

    print "Writing index for $range->{FIRST} -> $range->{LAST}\n" if ( $P{verbose} );

    my $dsc = '';
    if ( -f "$P{img_dir}/index.txt" ) {
	if ( open ( DSC, "$P{img_dir}/index.txt" ) ) {
	    print "Reading gallery description\n" if ( $P{verbose} );
	    while ( <DSC> ) {
		$dsc .= $_;
	    }
	    close ( DSC );
	}
	if ( $dsc ne '' ) {
	    $dsc = "</TR><TR><TD ALIGN=\"LEFT\" VALIGN=\"MIDDLE\">&nbsp;<BR>$dsc<BR>&nbsp;</TD></TR>";
	}
    }

    if ( open ( IND, ">$P{img_dir}/index_$this.html" ) ) {
	my $next_pg = $next != -1 ? "<A HREF=\"index_$next.html\">Next</A>"     : '&nbsp;';
	my $next_pg_hdr = $next != -1 ? "<link rel=\"next\" type=\"text/html\" HREF=\"index_$next.html\">" : ''; 
	my $prev_pg = $prev != -1 ? "<A HREF=\"index_$prev.html\">Previous</A>" : '&nbsp;';
	my $prev_pg_hdr = $prev != -1 ? "<link rel=\"prev\" type=\"text/html\" HREF=\"index_$prev.html\">" : ''; 
	my $ml = ( $P{metalink} ) ? "<A HREF=\"../index.html\">Gallery Index</A><P>" : '';
	my $ml_hdr = ( $P{metalink} ) ? "<link rel=\"start\" type=\"text/html\" HREF=\"../index.html\">" : '';

	my $position = '';
	if ( $this || $total ) {
	    $position = ($this+1) . ' / ' . ($total+1);
	}

	print IND<<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
$ml_hdr
$prev_pg_hdr
$next_pg_hdr
<TITLE>$P{title}</TITLE>
</HEAD>

<!--  Created by BBGallery $version                            -->
<!--  http://www.bb-zone.com/zope/bbzone/projects/bbgallery    -->

<BODY BGCOLOR="$P{bgcolor}" LINK="$P{link}" VLINK="$P{vlink}" ALINK="$P{alink}" TEXT="$P{text}">
<TABLE WIDTH="100%" COLS="3">
  <TR>
    <TD ALIGN="LEFT">$prev_pg</TD>
    <TD ALIGN="CENTER">$ml<BR>$position</TD>
    <TD ALIGN="RIGHT">$next_pg</TD>
  </TR>
</TABLE>
END
       if ( length ( $P{title} ) ) {
	   print IND<<END;
<HR>
<CENTER>$P{title}</CENTER>
END
       }
	print IND<<END;
<HR>
<CENTER>
<TABLE BGCOLOR="$P{bgcolor2}"><TR><TD ALIGN="CENTER">
<TABLE BGCOLOR="$P{bgcolor1}" CELLPADDING="5">
  <TR>
END
        for ( my $i=$range->{FIRST}; $i<=$range->{LAST}; $i++ ) {
	    my $image     = $img->[$i]->{THUMB};
	       $image     =~ s/^th_(.*)\.jpg/$1/;
	    my $image_cap = $image;
	    if ( ! $P{raw_captions} ) {
		$image_cap =~ s/_/ /g;
	    } else {
		$image_cap = $image_cap . '.jpg';
	    }
	    $image_cap    =  encode_entities($image_cap);
 	    my $image_uri = encode_entities(uri_escape($image));
 	    my $thumb_uri = encode_entities(uri_escape($img->[$i]->{THUMB}));
	    image_page ( $img, $i, $this, 0 );
	    image_page ( $img, $i, $this, 1 );
	    if ( $i - $range->{FIRST} && ($i - $range->{FIRST}) % $P{cols} == 0 ) {
		print IND "  </TR><TR>\n";
	    }

	    my $short = ''; 
	    if ( $P{short_dsc} ) {
		if ( -f "$P{img_dir}/$image.txt" ) {
		    if ( open ( DSC, "$P{img_dir}/$image.txt" ) ) {
			print "Reading image description\n" if ( $P{verbose} );
			$short = <DSC>;
			chomp($short);
			close ( DSC );
		    }
		}
		$short=~s/\<[^\>]*\>//g;
		$short = "ALT=\"$short\" TITLE=\"$short\"";
	    } else {
		$short = "ALT=\"$image_cap\"";
	    }

	    print IND<<END;
      <TD ALIGN="CENTER" VALIGN="MIDDLE">
	<A HREF="$image_uri.html"><IMG SRC="thumbs/$thumb_uri" $short BORDER="1"></A><BR>
        <FONT SIZE="-2">$image_cap</FONT>
      </TD>
END
	}
        print IND<<END;
  </TR>
</TABLE>
$dsc
</TD></TR></TABLE>
</CENTER>
<HR>
<TABLE WIDTH="100%" COLS="3">
  <TR>
    <TD ALIGN="LEFT">$prev_pg</TD>
    <TD ALIGN="CENTER">$position</TD>
    <TD ALIGN="RIGHT">$next_pg</TD>
  </TR>
</TABLE>
<CENTER>
<HR>
<FONT SIZE="-2">Created by <A HREF="http://www.bb-zone.com/zope/bbzone/projects/bbgallery">BBGallery</A></FONT>
</CENTER>
</BODY>
</HTML>
END
	close ( IND );
    }
}

sub parse_commandline () {
    my %c;

    # check for global config file
    if ( -f "$ENV{HOME}/.bbgallery" ) {
	load_config (  \%P, "$ENV{HOME}/.bbgallery" );
    }

    # config file in CWD?
    if ( -f ".bbgallery" ) {
	load_config (  \%P, ".bbgallery" );
    }

    return if ( $#ARGV < 0 );

    for ( my $i=0; $i<=$#ARGV; $i++ ) {
	for ( $ARGV[$i] ) {
	    ( /^-v$/ || /^--verbose$/ ) && do {
		$c{verbose} = ( defined ( $c{verbose} ) ) ? $c{verbose}++ : 1;
		last;
	    };
	    ( /^-f$/ || /^--force$/ )&& do {
		$c{force} = 1;
		last;
	    };
	    ( /^-s$/ || /^--scale$/ )&& do {
		$c{sc_size} = $ARGV[++$i];
		$c{scale} = ( $P{sc_size} == 0 ) ? 0 : 1;
		last;
	    };
	    /^--thumb$/ && do {
		$c{th_size} = $ARGV[++$i];
		last;
	    };
	    ( /^-c$/ || /^--cols$/ ) && do {
		$c{cols} = $ARGV[++$i];
		last;
	    };
	    ( /^-m$/ || /^--max$/ ) && do {
		$c{th_max} = $ARGV[++$i];
		last;
	    };
	    ( /^-t$/ || /^--title$/ ) && do {
		$c{title} =  encode_entities($ARGV[++$i]);
		last;
	    };
	    ( /^-d$/ || /^--directory$/ ) && do {
		$c{img_dir} = $ARGV[++$i];
		# config file in image dir?
		if ( -f "$c{img_dir}/.bbgallery" ) {
		    load_config ( \%P, "$c{img_dir}/.bbgallery" );
		}
		last;
	    };
	    ( /^-w$/ || /^--wait$/ ) && do {
		$c{slide_delay} = $ARGV[++$i];
		last;
	    };
	    ( /^-h$/ || /^--help$/ ) && do {
		usage();
		exit ( 1 );
		last;
	    };
	    ( /^-M$/ || /^--meta$/ ) && do {
		$c{meta} = 1;
		last;
	    };
	    ( /^-L$/ || /^--metalink$/ ) && do {
		$c{metalink} = 1;
		last;
	    };
	    ( /^-I$/ || /^--imagemagick$/ ) && do {
		$c{gimp} = 0;
		last;
	    };
	    ( /^-q$/ || /^--quality$/ ) && do {
		$c{quality} = $ARGV[++$i];
		last;
	    };
	    ( /^-V$/ || /^--version$/ ) && do {
		print STDERR "$0 Version $version\n";
		exit ( 1 );
	    };
	    ( /^--raw-caption$/ ) && do {
		$c{raw_captions} = 1;
		last;
	    };
	    ( /^--thumb-only$/ )&& do {
		$c{thumb_only} = 1;
		last;
	    };
	    ( /^--no-short-dsc/ )&& do {
		$c{short_dsc} = 0;
		last;
	    };
	    ( /^--no-save$/ ) && do {
		$c{no_save} = 1;
		last;
	    };
	    print "Ignoring unknown parameter: $ARGV[$i]\n";
	}
    }
    # make sure cmdline options take precedense over config files 
    # and builtin defaults
    map { $P{$_} = $c{$_} } keys ( %c );
}

sub usage () {
    print STDERR<<END;
$0 [options]
  Options:
    -v  --verbose         Be verbose [no]
    -f  --force           Force the (re)creation of thumb nails and
                          scaled images [no]
    -s  --scale <n>       Scale images to <n> pixels 
	                  (n=0 disables scaling) [800]
        --thumb <n>       Make thumbnails <n> pixels big [100]
        --thumb-only      Disable scaling, use thumbnails only [no]
        --no-short-dsc    Disable short descriptions
    -c  --cols  <n>       Create index pages with <n> columns [5]
    -m  --max   <n>       Maximal <n> thumb nails on one index
	                  page [25]
    -t  --title <s>       Use <s> as title for index pages
	                  [Gallery]
    -d  --directory <s>   Create gallery for images in directory <s> [.]
    -w  --wait <n>        Delay between slides in slide show is
                          <n> seconds [15]
    -M  --meta            Create meta index
    -h  --help            Show this help screen
    -L  --metalink        Enable link to meta index in gallerie [no]
    -V  --version         Print version number and exit
    -I  --ImageMagick     Use ImageMagick instead of Gimp to scale images
    -q  --quality <1-100> JPEG quality of scaled images [100]
        --raw-caption     Use filename as is for image caprions. By default 
                          underscores are replaced with spaces and the 
                          extension is stripped
        --no-save         Do not save configuration in image direcorty
END
}

# ----------------------------------------------------------------------
#                            M A I N
# ----------------------------------------------------------------------

# get parameters
parse_commandline ();

# normalize the inout directory to absolut
my $cwd;
chomp($cwd = `pwd`);
$P{img_dir} = "$cwd/$P{img_dir}" if (  $P{img_dir} !~ /^\// );

# save settings 
write_config ( "$P{img_dir}/.bbgallery" );

# meta index mode?
if ( $P{meta} ) {
    create_meta_index ();
    exit ( 0 );
}

# create thumbs and scale images 
scale_images() if ( $P{scale} );
create_thumbs();

# read in all the thumbs
if ( opendir ( TH, "$P{img_dir}/thumbs" ) ) {
    my @thumbs = sort ( readdir ( TH ) );
    closedir ( TH );
    my @img;
    my @idx;
    my $img_cnt = 0;
    my $idx_cnt = 0;
    my $first   = 0;

    # filter out all unwanted files and break the list down into index pages
    for ( my $i=0; $i<=$#thumbs; $i++ ) {	
	if ( $thumbs[$i] =~ /^th_.*\.jpg$/ ) {
	    if ( $img_cnt && ( $img_cnt % $P{th_max} == 0 ) ) {
		my $last = $img_cnt - 1;
		$idx[$idx_cnt++] = {
		    FIRST => $first,
		    LAST  => $img_cnt-1,
		};
		$first = $img_cnt;
	    }
	    $img[$img_cnt++] = {
		THUMB => $thumbs[$i],
		INDEX => $idx_cnt,
	    };
	}
    }
    $idx[$idx_cnt] = {
	FIRST => $first,
	LAST  => $img_cnt-1,
    };

    # create index pages
    for ( my $i=0; $i<=$#idx; $i++ ) {	
	index_page ( $idx[$i], \@img, 
		     ( $i ) ? $i-1: -1 ,                    # prev
		     $i,                                    # this
		     ( defined ( $idx[$i+1] ) ) ? $i+1: -1, # next
	             $#idx );                               # total
    }
    # write meta index entry
    meta_index_entry ( \@img );
}
exit ( 0 );
















